<?php
/**
 * Wholesale Pricing for WooCommerce - Shortcodes
 *
 * @version 1.1.2
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! function_exists( 'alg_wc_wholesale_pricing_table' ) ) {
	/**
	 * alg_wc_wholesale_pricing_table (global only).
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wholesale_pricing_table( $atts ) {
		// Shortcode atts
		$atts = shortcode_atts( array(
			'heading_format'        => sprintf( __( 'from %s pcs.', 'wholesale-pricing-woocommerce' ), '%level_min_qty%' ),
			'before_level_max_qty'  => '-',
			'last_level_max_qty'    => '+',
			'hide_if_zero_quantity' => 'no',
			'table_format'          => 'horizontal',
		), $atts, 'alg_wc_wholesale_pricing_table' );
		// Get levels data
		$wholesale_price_levels = alg_wc_wh_pr_get_levels_data( 0, false, 'asc' );
		// Form table data
		$data_qty       = array();
		$data_discount  = array();
		$columns_styles = array();
		$i              = -1;
		foreach ( $wholesale_price_levels as $wholesale_price_level ) {
			$i++;
			if ( 0 == $wholesale_price_level['quantity'] && 'yes' === $atts['hide_if_zero_quantity'] ) {
				continue;
			}
			// Quantity row
			$level_max_qty = ( isset( $wholesale_price_levels[ $i + 1 ]['quantity'] ) ) ?
				$atts['before_level_max_qty'] . ( $wholesale_price_levels[ $i + 1 ]['quantity'] - 1 ) : $atts['last_level_max_qty'];
			$replaced_values = array(
				'%level_min_qty%' => $wholesale_price_level['quantity'],
				'%level_max_qty%' => $level_max_qty,
			);
			$data_qty[]       = str_replace( array_keys( $replaced_values ), array_values( $replaced_values ), $atts['heading_format'] );
			// Discount row
			$data_discount[]  = ( 'fixed' === get_option( 'alg_wc_wholesale_pricing_discount_type', 'percent' ) )
				? '-' . wc_price( $wholesale_price_level['discount'] ) : '-' . $wholesale_price_level['discount'] . '%';
			// Column style
			$columns_styles[] = 'text-align: center;';
		}
		$table_rows = array( $data_qty, $data_discount );
		// Maybe switch rows and columns
		if ( 'vertical' === $atts['table_format'] ) {
			$table_rows_modified = array();
			foreach ( $table_rows as $row_number => $table_row ) {
				foreach ( $table_row as $column_number => $cell ) {
					$table_rows_modified[ $column_number ][ $row_number ] = $cell;
				}
			}
			$table_rows = $table_rows_modified;
		}
		// Get table HTML
		return alg_get_table_html( $table_rows,
			array( 'table_class' => 'alg_wc_wholesale_pricing_table', 'columns_styles' => $columns_styles, 'table_heading_type' => $atts['table_format'] ) );
	}
	// Adding shortcode
	add_shortcode( 'alg_wc_wholesale_pricing_table', 'alg_wc_wholesale_pricing_table' );
}

if ( ! function_exists( 'alg_wc_product_wholesale_pricing_table' ) ) {
	/**
	 * alg_wc_product_wholesale_pricing_table.
	 *
	 * @version 1.1.2
	 * @since   1.0.0
	 * @todo    [dev] (maybe) in "Percent row"  add for 'fixed'   or 'price_directly'
	 * @todo    [dev] (maybe) in "Discount row" add for 'percent' or 'price_directly'
	 */
	function alg_wc_product_wholesale_pricing_table( $atts ) {
		// Shortcode atts
		$atts = shortcode_atts( array(
			'product_id'            => 0,
			'heading_format'        => sprintf( __( 'from %s pcs.', 'wholesale-pricing-woocommerce' ), '%level_min_qty%' ),
			'before_level_max_qty'  => '-',
			'last_level_max_qty'    => '+',
			'hide_if_zero_quantity' => 'no',
			'table_format'          => 'horizontal',
			'hide_currency'         => 'no',
			'add_price_row'         => 'yes',
			'price_row_format'      => '<del>%old_price%</del> %price%',
			'add_percent_row'       => 'no',
			'add_discount_row'      => 'no',
		), $atts, 'alg_wc_product_wholesale_pricing_table' );
		// General
		$product    = ( 0 != $atts['product_id'] ? wc_get_product( $atts['product_id'] ) : wc_get_product() );
		$product_id = alg_wc_wh_pr_get_product_main_id( $product );
		if ( ! alg_wc_wh_pr_is_enabled( $product_id ) ) {
			return '';
		}
		// Get levels data
		$wholesale_price_levels = alg_wc_wh_pr_get_levels_data( $product_id, null, 'asc' );
		// Discount type
		$discount_type = alg_wc_wh_pr_get_discount_type( $product_id );
		// Form table data
		$data_qty       = array();
		$data_price     = array();
		$data_discount  = array();
		$columns_styles = array();
		$i              = -1;
		foreach ( $wholesale_price_levels as $wholesale_price_level ) {
			$i++;
			if ( 0 == $wholesale_price_level['quantity'] && 'yes' === $atts['hide_if_zero_quantity'] ) {
				continue;
			}
			// Quantity row
			$level_max_qty = ( isset( $wholesale_price_levels[ $i + 1 ]['quantity'] ) ) ?
				$atts['before_level_max_qty'] . ( $wholesale_price_levels[ $i + 1 ]['quantity'] - 1 ) : $atts['last_level_max_qty'];
			$replaced_values = array(
				'%level_min_qty%' => $wholesale_price_level['quantity'],
				'%level_max_qty%' => $level_max_qty,
			);
			$data_qty[] = str_replace( array_keys( $replaced_values ), array_values( $replaced_values ), $atts['heading_format'] );
			// Price row
			if ( 'yes' === $atts['add_price_row'] ) {
				$data_price[] = alg_wc_wh_pr_get_product_price( $product,
					$discount_type, $wholesale_price_level['discount'], $atts['hide_currency'], $atts['price_row_format'] );
			}
			// Percent row
			if ( 'yes' === $atts['add_percent_row'] ) {
				if ( 'percent' === $discount_type ) {
					$data_discount[] = '-' . $wholesale_price_level['discount'] . '%';
				}
			}
			// Discount row
			if ( 'yes' === $atts['add_discount_row'] ) {
				if ( 'fixed' === $discount_type ) {
					$data_discount[] = '-' . wc_price( $wholesale_price_level['discount'] );
				}
			}
			// Column style
			$columns_styles[] = 'text-align: center;';
		}
		// Table rows
		$table_rows = array( $data_qty );
		if ( 'yes' === $atts['add_price_row'] ) {
			$table_rows[] = $data_price;
		}
		if ( 'yes' === $atts['add_percent_row'] ) {
			$table_rows[] = $data_discount;
		}
		// Maybe switch rows and columns
		if ( 'vertical' === $atts['table_format'] ) {
			$table_rows_modified = array();
			foreach ( $table_rows as $row_number => $table_row ) {
				foreach ( $table_row as $column_number => $cell ) {
					$table_rows_modified[ $column_number ][ $row_number ] = $cell;
				}
			}
			$table_rows = $table_rows_modified;
		}
		// Get table HTML
		return alg_get_table_html( $table_rows,
			array( 'table_class' => 'alg_wc_product_wholesale_pricing_table', 'columns_styles' => $columns_styles, 'table_heading_type' => $atts['table_format'] ) );
	}
	// Adding shortcode
	add_shortcode( 'alg_wc_product_wholesale_pricing_table', 'alg_wc_product_wholesale_pricing_table' );
}

if ( ! function_exists( 'alg_wc_wholesale_pricing_data' ) ) {
	/**
	 * alg_wc_wholesale_pricing_data (global only).
	 *
	 * @version 1.1.2
	 * @since   1.1.2
	 */
	function alg_wc_wholesale_pricing_data( $atts ) {
		$atts = shortcode_atts( array(
			'field'     => 'discount', // or 'quantity'
			'level_num' => 'last',     // or actual level num
		), $atts, 'alg_wc_wholesale_pricing_data' );
		$wholesale_price_levels = alg_wc_wh_pr_get_levels_data( 0, false, 'asc' );
		$level_num              = ( 'last' === $atts['level_num'] ? count( $wholesale_price_levels ) : $atts['level_num'] ) - 1;
		return ( isset( $wholesale_price_levels[ $level_num ][ $atts['field'] ] ) ? $wholesale_price_levels[ $level_num ][ $atts['field'] ] : '' );
	}
	// Adding shortcode
	add_shortcode( 'alg_wc_wholesale_pricing_data', 'alg_wc_wholesale_pricing_data' );
}

if ( ! function_exists( 'alg_wc_product_wholesale_pricing_data' ) ) {
	/**
	 * alg_wc_product_wholesale_pricing_data.
	 *
	 * @version 1.1.2
	 * @since   1.1.2
	 */
	function alg_wc_product_wholesale_pricing_data( $atts ) {
		$atts = shortcode_atts( array(
			'product_id'            => 0,
			'price_format'          => '<del>%old_price%</del> %price%',
			'field'                 => 'price', // or 'discount' or 'quantity'
			'level_num'             => 'last',  // or actual level num
			'hide_currency'         => 'no',
		), $atts, 'alg_wc_product_wholesale_pricing_data' );
		$product    = ( 0 != $atts['product_id'] ? wc_get_product( $atts['product_id'] ) : wc_get_product() );
		$product_id = alg_wc_wh_pr_get_product_main_id( $product );
		if ( ! alg_wc_wh_pr_is_enabled( $product_id ) ) {
			return '';
		}
		$wholesale_price_levels = alg_wc_wh_pr_get_levels_data( $product_id, null, 'asc' );
		$level_num              = ( 'last' === $atts['level_num'] ? count( $wholesale_price_levels ) : $atts['level_num'] ) - 1;
		if ( isset( $wholesale_price_levels[ $level_num ] ) ) {
			switch ( $atts['field'] ) {
				case 'quantity':
				case 'discount':
					return ( isset( $wholesale_price_levels[ $level_num ][ $atts['field'] ] ) ? $wholesale_price_levels[ $level_num ][ $atts['field'] ] : '' );
				default: // 'price':
					return alg_wc_wh_pr_get_product_price( $product,
						alg_wc_wh_pr_get_discount_type( $product_id ), $wholesale_price_levels[ $level_num ]['discount'], $atts['hide_currency'], $atts['price_format'] );
			}
		} else {
			return '';
		}
	}
	// Adding shortcode
	add_shortcode( 'alg_wc_product_wholesale_pricing_data', 'alg_wc_product_wholesale_pricing_data' );
}
