<?php
/**
 * Wholesale Pricing for WooCommerce - Core Functions
 *
 * @version 1.0.0
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! function_exists( 'alg_wc_wh_pr_get_discount_by_quantity' ) ) {
	/**
	 * alg_wc_wh_pr_get_discount_by_quantity.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_discount_by_quantity( $quantity, $product_id ) {
		foreach ( alg_wc_wh_pr_get_levels_data( $product_id ) as $level_data ) {
			if ( $quantity >= $level_data['quantity'] ) {
				return $level_data['discount'];
			}
		}
		return 0;
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_wholesale_price' ) ) {
	/**
	 * alg_wc_wh_pr_get_wholesale_price.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_wholesale_price( $price, $quantity, $product_id ) {
		$discount = alg_wc_wh_pr_get_discount_by_quantity( $quantity, $product_id );
		switch ( alg_wc_wh_pr_get_discount_type( $product_id ) ) {
			case 'price_directly':
				return ( 0 != $discount ) ? $discount : $price;
			case 'percent':
				return $price * ( 1.0 - ( $discount / 100.0 ) );
			default: // 'fixed'
				$discounted_price = $price - $discount;
				return ( $discounted_price >= 0 ) ? $discounted_price : 0;
		}
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_discount_type' ) ) {
	/**
	 * alg_wc_wh_pr_get_discount_type.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_discount_type( $product_id ) {
		return ( alg_wc_wh_pr_is_enabled_per_product( $product_id ) ?
			get_post_meta( $product_id, '_' . 'alg_wc_wholesale_pricing_discount_type', true ) :
			get_option( 'alg_wc_wholesale_pricing_discount_type', 'percent' )
		);
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_sort_levels_data_by_quantity_asc' ) ) {
	/**
	 * alg_wc_wh_pr_sort_levels_data_by_quantity_asc.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_sort_levels_data_by_quantity_asc( $a, $b ) {
		if ( $a['quantity'] == $b['quantity'] ) {
			return 0;
		}
		return ( $a['quantity'] < $b['quantity'] ) ? -1 : 1;
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_sort_levels_data_by_quantity_desc' ) ) {
	/**
	 * alg_wc_wh_pr_sort_levels_data_by_quantity_desc.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_sort_levels_data_by_quantity_desc( $a, $b ) {
		if ( $a['quantity'] == $b['quantity'] ) {
			return 0;
		}
		return ( $a['quantity'] < $b['quantity'] ) ? 1 : -1;
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_levels_data' ) ) {
	/**
	 * alg_wc_wh_pr_get_levels_data.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_levels_data( $product_id, $is_per_product = null, $sort = 'desc' ) {
		$role_option_name_addon = alg_wc_wh_pr_get_user_role_option_name_addon();
		$is_per_product         = ( null === $is_per_product ? alg_wc_wh_pr_is_enabled_per_product( $product_id ) : $is_per_product );
		$levels_data            = array();
		for ( $level = 1; $level <= alg_wc_wh_pr_get_total_levels( $is_per_product, $product_id, $role_option_name_addon ); $level++ ) {
			$levels_data[] = array(
				'quantity' => alg_wc_wh_pr_get_level_quantity( $is_per_product, $level, $product_id, $role_option_name_addon ),
				'discount' => alg_wc_wh_pr_get_level_discount( $is_per_product, $level, $product_id, $role_option_name_addon ),
			);
		}
		usort( $levels_data, 'alg_wc_wh_pr_sort_levels_data_by_quantity_' . $sort );
		return $levels_data;
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_level_discount' ) ) {
	/**
	 * alg_wc_wh_pr_get_level_discount.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_level_discount( $is_per_product, $level, $product_id, $role_option_name_addon ) {
		return ( $is_per_product ?
			get_post_meta( $product_id, '_' . 'alg_wc_wholesale_pricing_level_discount' . $role_option_name_addon . '_' . $level, true ) :
			get_option( 'alg_wc_wholesale_pricing_level_discount' . $role_option_name_addon . '_' . $level, 0 )
		);
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_level_quantity' ) ) {
	/**
	 * alg_wc_wh_pr_get_level_quantity.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_level_quantity( $is_per_product, $level, $product_id, $role_option_name_addon ) {
		return ( $is_per_product ?
			get_post_meta( $product_id, '_' . 'alg_wc_wholesale_pricing_level_min_qty' . $role_option_name_addon . '_' . $level, true ) :
			get_option( 'alg_wc_wholesale_pricing_level_min_qty' . $role_option_name_addon . '_' . $level, 0 )
		);
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_total_levels' ) ) {
	/**
	 * alg_wc_wh_pr_get_total_levels.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_total_levels( $is_per_product, $product_id = 0, $role_option_name_addon = '' ) {
		return ( $is_per_product ?
			apply_filters( 'alg_wc_wholesale_pricing', 1, 'value_levels_per_product', array( 'product_id' => $product_id, 'role_option_name_addon' => $role_option_name_addon ) ) :
			apply_filters( 'alg_wc_wholesale_pricing', 1, 'value_levels', array( 'role_option_name_addon' => $role_option_name_addon ) )
		);
	}
}
