<?php
/**
 * Wholesale Pricing for WooCommerce - Products Functions
 *
 * @version 1.1.2
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! function_exists( 'alg_wc_wh_pr_get_product_main_id' ) ) {
	/**
	 * alg_wc_wh_pr_get_product_main_id.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_product_main_id( $_product ) {
		return ( $_product->is_type( 'variation' ) ) ? $_product->get_parent_id() : $_product->get_id();
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_products' ) ) {
	/**
	 * alg_wc_wh_pr_get_products.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function alg_wc_wh_pr_get_products( $products = array(), $post_status = 'any', $block_size = 512, $add_variations = false ) {
		$offset = 0;
		while( true ) {
			$args = array(
				'post_type'      => 'product',
				'post_status'    => $post_status,
				'posts_per_page' => $block_size,
				'offset'         => $offset,
				'orderby'        => 'title',
				'order'          => 'ASC',
				'fields'         => 'ids',
			);
			$loop = new WP_Query( $args );
			if ( ! $loop->have_posts() ) {
				break;
			}
			foreach ( $loop->posts as $post_id ) {
				$products[ $post_id ] = get_the_title( $post_id );
				if ( $add_variations ) {
					$_product = wc_get_product( $post_id );
					if ( $_product->is_type( 'variable' ) ) {
						foreach ( $_product->get_children() as $child_id ) {
							$products[ $child_id ] = get_the_title( $child_id );
						}
					}
				}
			}
			$offset += $block_size;
		}
		return $products;
	}
}

if ( ! function_exists( 'alg_wc_wh_pr_get_product_price' ) ) {
	/**
	 * alg_wc_wh_pr_get_product_price.
	 *
	 * @version 1.1.2
	 * @since   1.1.2
	 */
	function alg_wc_wh_pr_get_product_price( $product, $discount_type, $discount, $hide_currency, $price_format ) {
		$price          = '';
		$price_original = '';
		if ( $product->is_type( 'variable' ) ) {
			// Variable
			$prices       = $product->get_variation_prices( false );
			$min_key      = key( $prices['price'] );
			end( $prices['price'] );
			$max_key      = key( $prices['price'] );
			$min_product  = wc_get_product( $min_key );
			$max_product  = wc_get_product( $max_key );
			$min          = wc_get_price_to_display( $min_product );
			$max          = wc_get_price_to_display( $max_product );
			$min_original = $min;
			$max_original = $max;
			if ( 'fixed' === $discount_type ) {
				$min  = $min - $discount;
				$max  = $max - $discount;
			} else {
				$coef = 1.0 - ( $discount / 100.0 );
				$min  = $min * $coef;
				$max  = $max * $coef;
			}
			if ( 'yes' !== $hide_currency ) {
				$min          = wc_price( $min );
				$max          = wc_price( $max );
				$min_original = wc_price( $min_original );
				$max_original = wc_price( $max_original );
			}
			$price          = ( $min          != $max          ? sprintf( '%s-%s', $min,          $max )          : $min );
			$price_original = ( $min_original != $max_original ? sprintf( '%s-%s', $min_original, $max_original ) : $min_original );
		} else {
			// Simple etc.
			$price          = wc_get_price_to_display( $product );
			$price_original = $price;
			if ( 'price_directly' === $discount_type ) {
				$price = $discount;
			} elseif ( 'fixed' === $discount_type ) {
				$price = $price - $discount;
			} else { // 'percent'
				$coef  = 1.0 - ( $discount / 100.0 );
				$price = $price * $coef;
			}
			if ( 'yes' !== $hide_currency ) {
				$price          = wc_price( $price );
				$price_original = wc_price( $price_original );
			}
		}
		return str_replace( array( '%old_price%', '%price%' ), array( $price_original, $price ), $price_format );
	}
}
