<?php
/**
 * Wholesale Pricing for WooCommerce - Metaboxes
 *
 * @version 1.1.0
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Wholesale_Pricing_Metaboxes' ) ) :

class Alg_WC_Wholesale_Pricing_Metaboxes {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function __construct() {
		add_action( 'add_meta_boxes',    array( $this, 'add_wholesale_pricing_metabox' ) );
		add_action( 'save_post_product', array( $this, 'save_wholesale_pricing_meta_box' ), PHP_INT_MAX, 2 );
	}

	/**
	 * add_wholesale_pricing_metabox.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function add_wholesale_pricing_metabox() {
		add_meta_box(
			'alg-wholesale-pricing',
			__( 'Wholesale Pricing', 'wholesale-pricing-woocommerce' ),
			array( $this, 'display_wholesale_pricing_metabox' ),
			'product',
			'normal',
			'high'
		);
	}

	/**
	 * display_wholesale_pricing_metabox.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) placeholder for textarea
	 */
	function display_wholesale_pricing_metabox() {
		$current_post_id = get_the_ID();
		$html = '';
		$html .= '<table class="widefat striped">';
		foreach ( $this->get_meta_box_options() as $option ) {
			$is_enabled = ( isset( $option['enabled'] ) && 'no' === $option['enabled'] ) ? false : true;
			if ( $is_enabled ) {
				if ( 'title' === $option['type'] ) {
					$html .= '<tr>';
					$html .= '<th colspan="3" style="text-align:left;font-weight:bold;">' . $option['title'] . '</th>';
					$html .= '</tr>';
				} else {
					$custom_attributes = '';
					$the_post_id   = ( isset( $option['product_id'] ) ) ? $option['product_id'] : $current_post_id;
					$the_meta_name = ( isset( $option['meta_name'] ) )  ? $option['meta_name']  : '_' . $option['name'];
					if ( get_post_meta( $the_post_id, $the_meta_name ) ) {
						$option_value = get_post_meta( $the_post_id, $the_meta_name, true );
					} else {
						$option_value = ( isset( $option['default'] ) ) ? $option['default'] : '';
					}
					$css = ( isset( $option['css'] ) ) ? $option['css']  : '';
					$input_ending = '';
					if ( 'select' === $option['type'] ) {
						if ( isset( $option['multiple'] ) ) {
							$custom_attributes = ' multiple';
							$option_name       = $option['name'] . '[]';
						} else {
							$option_name       = $option['name'];
						}
						if ( isset( $option['custom_attributes'] ) ) {
							$custom_attributes .= ' ' . $option['custom_attributes'];
						}
						$options = '';
						foreach ( $option['options'] as $select_option_key => $select_option_value ) {
							$selected = '';
							if ( is_array( $option_value ) ) {
								foreach ( $option_value as $single_option_value ) {
									if ( '' != ( $selected = selected( $single_option_value, $select_option_key, false ) ) ) {
										break;
									}
								}
							} else {
								$selected = selected( $option_value, $select_option_key, false );
							}
							$options .= '<option value="' . $select_option_key . '" ' . $selected . '>' . $select_option_value . '</option>';
						}
					} elseif ( 'textarea' === $option['type'] ) {
						if ( '' === $css ) {
							$css = 'min-width:300px;';
						}
					} else {
						$input_ending = ' id="' . $option['name'] . '" name="' . $option['name'] . '" value="' . $option_value . '">';
						if ( isset( $option['custom_attributes'] ) ) {
							$input_ending = ' ' . $option['custom_attributes'] . $input_ending;
						}
						if ( isset( $option['placeholder'] ) ) {
							$input_ending = ' placeholder="' . $option['placeholder'] . '"' . $input_ending;
						}
					}
					switch ( $option['type'] ) {
						case 'price':
							$field_html = '<input style="' . $css . '" class="short wc_input_price" type="number" step="0.0001"' . $input_ending;
							break;
						case 'date':
							$field_html = '<input style="' . $css . '" class="input-text" display="date" type="text"' . $input_ending;
							break;
						case 'textarea':
							$field_html = '<textarea style="' . $css . '" id="' . $option['name'] . '" name="' . $option['name'] . '">' . $option_value . '</textarea>';
							break;
						case 'select':
							$field_html = '<select' . $custom_attributes . ' style="' . $css . '" id="' . $option['name'] . '" name="' . $option_name . '">' . $options . '</select>';
							break;
						default:
							$field_html = '<input style="' . $css . '" class="short" type="' . $option['type'] . '"' . $input_ending;
							break;
					}
					$html .= '<tr>';
					$maybe_tooltip = ( isset( $option['tooltip'] ) && '' != $option['tooltip'] ) ? wc_help_tip( $option['tooltip'], true ) : '';
					$html .= '<th style="text-align:left;width:25%;">' . $option['title'] . $maybe_tooltip . '</th>';
					if ( isset( $option['desc'] ) && '' != $option['desc'] ) {
						$html .= '<td style="font-style:italic;width:25%;">' . $option['desc'] . '</td>';
					}
					$html .= '<td>' . $field_html . '</td>';
					$html .= '</tr>';
				}
			}
		}
		$html .= '</table>';
		$html .= '<input type="hidden" name="alg_wc_wholesale_pricing_save_post" value="alg_wc_wholesale_pricing_save_post">';
		echo $html;
	}

	/**
	 * save_wholesale_pricing_meta_box.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function save_wholesale_pricing_meta_box( $post_id, $post ) {
		// Check that we are saving with current metabox displayed.
		if ( ! isset( $_POST[ 'alg_wc_wholesale_pricing_save_post' ] ) ) {
			return;
		}
		// Save options
		foreach ( $this->get_meta_box_options() as $option ) {
			if ( 'title' === $option['type'] ) {
				continue;
			}
			$is_enabled = ( isset( $option['enabled'] ) && 'no' === $option['enabled'] ) ? false : true;
			if ( $is_enabled ) {
				$option_value  = ( isset( $_POST[ $option['name'] ] ) ? $_POST[ $option['name'] ] : $option['default'] );
				$_post_id      = ( isset( $option['product_id'] )     ? $option['product_id']     : $post_id );
				$_meta_name    = ( isset( $option['meta_name'] )      ? $option['meta_name']      : '_' . $option['name'] );
				update_post_meta( $_post_id, $_meta_name, $option_value );
			}
		}
	}

	/**
	 * get_meta_box_options.
	 *
	 * @version 1.1.0
	 * @since   1.0.0
	 * @todo    [dev] (maybe) restyle etc.
	 */
	function get_meta_box_options() {
		$product_id = get_the_ID();
		$_product   = wc_get_product( $product_id );
		if ( ! $_product ) {
			return array();
		}
		$discount_type_options = array(
			'percent'        => __( 'Percent', 'wholesale-pricing-woocommerce' ),
			'fixed'          => __( 'Fixed', 'wholesale-pricing-woocommerce' ),
			'price_directly' => __( 'Price directly', 'wholesale-pricing-woocommerce' ),
		);
		if ( $_product->is_type( 'variable' ) ) {
			unset( $discount_type_options['price_directly'] );
		}
		$options = array(
			array(
				'name'     => 'alg_wc_wholesale_pricing_per_product_enabled',
				'default'  => 'no',
				'type'     => 'select',
				'options'  => array(
					'yes' => __( 'Yes', 'wholesale-pricing-woocommerce' ),
					'no'  => __( 'No', 'wholesale-pricing-woocommerce' ),
				),
				'title'    => __( 'Enable per product levels', 'wholesale-pricing-woocommerce' ),
			),
			array(
				'name'     => 'alg_wc_wholesale_pricing_discount_type',
				'default'  => 'percent',
				'type'     => 'select',
				'options'  => $discount_type_options,
				'title'    => __( 'Discount type', 'wholesale-pricing-woocommerce' ),
				'tooltip'  => __( '"Price directly" option is only available for simple (i.e. non variable) product type.', 'wholesale-pricing-woocommerce' ),
			),
			array(
				'name'     => 'alg_wc_wholesale_pricing_levels_number',
				'default'  => 0,
				'type'     => 'number',
				'title'    => __( 'Number of levels', 'wholesale-pricing-woocommerce' ),
				'tooltip'  => __( 'Save product after you change this number (new settings fields will be shown).', 'wholesale-pricing-woocommerce' ) .
					apply_filters( 'alg_wc_wholesale_pricing', ' ' . sprintf( __( 'Free version is limited to one level maximum. Please visit %s to get Pro version.', 'wholesale-pricing-woocommerce' ), 'WPFactory.com' ), 'settings' ),
				'custom_attributes' => 'min="0" max="' . apply_filters( 'alg_wc_wholesale_pricing', 1, 'max_levels_per_product' ) . '"',
			),
		);
		$discount_type = get_post_meta( $product_id, '_' . 'alg_wc_wholesale_pricing_discount_type', true );
		for ( $i = 1; $i <= alg_wc_wh_pr_get_total_levels( true, $product_id ); $i++ ) {
			$options = array_merge( $options, array(
				array(
					'type'    => 'title',
					'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i,
				),
				array(
					'name'    => 'alg_wc_wholesale_pricing_level_min_qty_' . $i,
					'default' => 0,
					'type'    => 'number',
					'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' ' . __( 'min quantity', 'wholesale-pricing-woocommerce' ),
					'custom_attributes' => 'min="0"',
					'tooltip' => __( 'Minimum quantity to apply discount.', 'wholesale-pricing-woocommerce' ),
				),
				array(
					'name'    => 'alg_wc_wholesale_pricing_level_discount_' . $i,
					'default' => 0,
					'type'    => 'price',
					'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' ' .
						( 'price_directly' === $discount_type ? __( 'price', 'wholesale-pricing-woocommerce' ) : __( 'discount', 'wholesale-pricing-woocommerce' ) ),
					'tooltip' => ( 'price_directly' === $discount_type ? '' :
						__( 'To set fee instead of discount - enter negative number.', 'wholesale-pricing-woocommerce' ) ),
				),
			) );
		}
		$user_roles = get_option( 'alg_wc_wholesale_pricing_by_user_role_roles', '' );
		if ( ! empty( $user_roles ) ) {
			foreach ( $user_roles as $user_role_key ) {
				$options = array_merge( $options, array(
					array(
						'type'    => 'title',
						'title'   => $user_role_key,
					),
					array(
						'name'    => 'alg_wc_wholesale_pricing_levels_number_' . $user_role_key,
						'default' => 0,
						'type'    => 'number',
						'title'   => __( 'Number of levels', 'wholesale-pricing-woocommerce' ) . ' [' . $user_role_key . ']',
						'tooltip'  => __( 'Save product after you change this number (new settings fields will be shown).', 'wholesale-pricing-woocommerce' ) .
							apply_filters( 'alg_wc_wholesale_pricing', ' ' . sprintf( __( 'Free version is limited to one level maximum. Please visit %s to get Pro version.', 'wholesale-pricing-woocommerce' ), 'WPFactory.com' ), 'settings' ),
						'custom_attributes' => 'min="0" max="' . apply_filters( 'alg_wc_wholesale_pricing', 1, 'max_levels_per_product' ) . '"',
					),
				) );
				for ( $i = 1; $i <= alg_wc_wh_pr_get_total_levels( true, $product_id, '_' . $user_role_key ); $i++ ) {
					$options = array_merge( $options, array(
						array(
							'type'    => 'title',
							'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i,
						),
						array(
							'name'    => 'alg_wc_wholesale_pricing_level_min_qty_' . $user_role_key . '_' . $i,
							'default' => 0,
							'type'    => 'number',
							'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' ' .
								__( 'min quantity', 'wholesale-pricing-woocommerce' ) . ' [' . $user_role_key . ']',
							'custom_attributes' => 'min="0"',
							'tooltip' => __( 'Minimum quantity to apply discount.', 'wholesale-pricing-woocommerce' ),
						),
						array(
							'name'    => 'alg_wc_wholesale_pricing_level_discount_' . $user_role_key . '_' . $i,
							'default' => 0,
							'type'    => 'price',
							'title'   => __( 'Level', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' ' .
								( 'price_directly' === $discount_type ? __( 'price', 'wholesale-pricing-woocommerce' ) : __( 'discount', 'wholesale-pricing-woocommerce' ) )  . ' [' . $user_role_key . ']',
							'tooltip' => ( 'price_directly' === $discount_type ? '' :
								__( 'To set fee instead of discount - enter negative number.', 'wholesale-pricing-woocommerce' ) ),
						),
					) );
				}
			}
		}
		return $options;
	}

}

endif;

return new Alg_WC_Wholesale_Pricing_Metaboxes();
