<?php
/**
 * Wholesale Pricing for WooCommerce - General Section Settings
 *
 * @version 1.1.2
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'Alg_WC_Wholesale_Pricing_Settings_General' ) ) :

class Alg_WC_Wholesale_Pricing_Settings_General extends Alg_WC_Wholesale_Pricing_Settings_Section {

	/**
	 * Constructor.
	 *
	 * @version 1.0.0
	 * @since   1.0.0
	 */
	function __construct() {
		$this->id   = '';
		$this->desc = __( 'General', 'wholesale-pricing-woocommerce' );
		parent::__construct();
	}

	/**
	 * get_settings.
	 *
	 * @version 1.1.2
	 * @since   1.0.0
	 * @todo    [fix] (maybe) fix cart info, when discount < 0
	 * @todo    [dev] (maybe) restyle
	 * @todo    [dev] (maybe) split into separate sections: General, Wholesale Levels (and maybe User Roles)
	 */
	function get_settings() {
		$main_settings = array(
			array(
				'title'    => __( 'Wholesale Pricing Options', 'wholesale-pricing-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_wholesale_pricing_options',
			),
			array(
				'title'    => __( 'Wholesale Pricing for WooCommerce', 'wholesale-pricing-woocommerce' ),
				'desc'     => '<strong>' . __( 'Enable plugin', 'wholesale-pricing-woocommerce' ) . '</strong>',
				'id'       => 'alg_wc_wholesale_pricing_enabled',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_wholesale_pricing_options',
			),
		);
		$products = alg_wc_wh_pr_get_products();
		$general_settings = array(
			array(
				'title'    => __( 'General', 'wholesale-pricing-woocommerce' ),
				'type'     => 'title',
				'desc'     => sprintf( __( 'If you want to display prices table on frontend, use %s shortcodes.', 'wholesale-pricing-woocommerce' ),
					'<code>[alg_wc_wholesale_pricing_table]</code>, <code>[alg_wc_product_wholesale_pricing_table]</code>' ) . '<br>' .
					sprintf( __( 'If you want to display other wholesale pricing info on frontend, use %s shortcodes.', 'wholesale-pricing-woocommerce' ),
					'<code>[alg_wc_wholesale_pricing_data]</code>, <code>[alg_wc_product_wholesale_pricing_data]</code>' ),
				'id'       => 'alg_wc_wholesale_pricing_general_options',
			),
			array(
				'title'    => __( 'Enable per product', 'wholesale-pricing-woocommerce' ),
				'desc'     => __( 'Enable', 'wholesale-pricing-woocommerce' ),
				'desc_tip' => __( 'This will add new meta box to each product\'s edit page.', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_per_product_enabled',
				'default'  => 'yes',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Use total cart quantity instead of product quantity', 'wholesale-pricing-woocommerce' ),
				'desc'     => __( 'Enable', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_use_total_cart_quantity',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Apply wholesale discount only if no other cart discounts were applied', 'wholesale-pricing-woocommerce' ),
				'desc'     => __( 'Enable', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_apply_only_if_no_other_discounts',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'title'    => __( 'Show discount info on cart page', 'wholesale-pricing-woocommerce' ),
				'desc'     => __( 'Show', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_show_info_on_cart',
				'default'  => 'no',
				'type'     => 'checkbox',
			),
			array(
				'desc_tip' => __( 'If show discount info on cart page is enabled, set format here.', 'wholesale-pricing-woocommerce' ) . ' ' .
					__( 'Replaced values: %old_price%, %price%, %discount_value%.', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_show_info_on_cart_format',
				'default'  => '<del>%old_price%</del> %price%<br>You save: <span style="color:red;">%discount_value%</span>',
				'type'     => 'textarea',
				'css'      => 'width:100%;',
				'alg_wc_wh_pr_raw' => true,
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_wholesale_pricing_general_options',
			),
			array(
				'title'    => __( 'Discount', 'wholesale-pricing-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_wholesale_pricing_discount_options',
			),
			array(
				'title'    => __( 'Discount type', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_discount_type',
				'default'  => 'percent',
				'type'     => 'select',
				'options'  => array(
					'percent' => __( 'Percent', 'wholesale-pricing-woocommerce' ),
					'fixed'   => __( 'Fixed', 'wholesale-pricing-woocommerce' ),
				),
			),
			array(
				'title'    => __( 'Products to include', 'wholesale-pricing-woocommerce' ),
				'desc_tip' => __( 'Leave blank to include all products.', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_products_to_include',
				'default'  => '',
				'type'     => 'multiselect',
				'class'    => 'chosen_select',
				'options'  => $products,
			),
			array(
				'title'    => __( 'Products to exclude', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_products_to_exclude',
				'default'  => '',
				'type'     => 'multiselect',
				'class'    => 'chosen_select',
				'options'  => $products,
			),
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_wholesale_pricing_discount_options',
			),
		);
		$levels_settings = array(
			array(
				'title'    => __( 'Wholesale Table', 'wholesale-pricing-woocommerce' ),
				'type'     => 'title',
				'id'       => 'alg_wc_wholesale_pricing_level_options',
			),
			array(
				'title'    => __( 'Number of levels', 'wholesale-pricing-woocommerce' ),
				'desc_tip' => __( 'Save settings after you change this option (new settings fields will be shown).', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_levels_number',
				'default'  => 1,
				'type'     => 'number',
				'desc'     => apply_filters( 'alg_wc_wholesale_pricing', sprintf(
					'<p>You will need <a target="_blank" href="%s">Wholesale Pricing for WooCommerce Pro</a> plugin to add more than one level.</p>',
					'https://wpfactory.com/item/wholesale-pricing-woocommerce/' ), 'settings_number_of_levels' ),
				'custom_attributes' => apply_filters( 'alg_wc_wholesale_pricing', array( 'readonly' => 'readonly' ), 'settings_array' ),
			),
		);
		for ( $i = 1; $i <= alg_wc_wh_pr_get_total_levels( false ); $i++ ) {
			$levels_settings = array_merge( $levels_settings, array(
				array(
					'title'    => __( 'Min quantity', 'wholesale-pricing-woocommerce' ) . ' #' . $i,
					'desc_tip' => __( 'Minimum quantity to apply discount.', 'wholesale-pricing-woocommerce' ),
					'id'       => 'alg_wc_wholesale_pricing_level_min_qty_' . $i,
					'default'  => 0,
					'type'     => 'number',
					'custom_attributes' => array('step' => '1', 'min' => '0', ),
				),
				array(
					'title'    => __( 'Discount', 'wholesale-pricing-woocommerce' ) . ' #' . $i,
					'desc_tip' => __( 'To set fee instead of discount - enter negative number.', 'wholesale-pricing-woocommerce' ),
					'id'       => 'alg_wc_wholesale_pricing_level_discount_' . $i,
					'default'  => 0,
					'type'     => 'number',
					'custom_attributes' => array('step' => '0.0001' ),
				),
			) );
		}
		$levels_settings = array_merge( $levels_settings, array(
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_wholesale_pricing_level_options',
			),
		) );
		$user_roles_settings = array(
			array(
				'title'    => __( 'Additional User Roles', 'wholesale-pricing-woocommerce' ),
				'type'     => 'title',
				'desc'     => __( 'If you want to set different wholesale pricing options for different user roles, fill this section.', 'wholesale-pricing-woocommerce' ),
				'id'       => 'alg_wc_wholesale_pricing_by_user_role_options',
			),
			array(
				'title'    => __( 'User roles settings', 'wholesale-pricing-woocommerce' ),
				'desc_tip' => __( 'Save settings after you change this option (new settings fields will be shown).', 'wholesale-pricing-woocommerce' ) . ' ' .
					__( 'Leave blank to disable.', 'wholesale-pricing-woocommerce' ),
				'type'     => 'multiselect',
				'id'       => 'alg_wc_wholesale_pricing_by_user_role_roles',
				'default'  => '',
				'class'    => 'chosen_select',
				'options'  => alg_wc_wh_pr_get_user_roles_options(),
				'desc'     => '<button name="save" class="button-primary woocommerce-save-button" type="submit" value="' . esc_attr( __( 'Save changes', 'woocommerce' ) ) . '">' . esc_html( __( 'Save changes', 'woocommerce' ) ) . '</button>',
			),
		);
		$user_roles = get_option( 'alg_wc_wholesale_pricing_by_user_role_roles', '' );
		if ( ! empty( $user_roles ) ) {
			foreach ( $user_roles as $user_role_key ) {
				$user_roles_settings = array_merge( $user_roles_settings, array(
					array(
						'title'    => __( 'Number of levels', 'wholesale-pricing-woocommerce' ) . ' [' . $user_role_key . ']',
						'desc_tip' => __( 'Save settings after you change this option (new settings fields will be shown).', 'wholesale-pricing-woocommerce' ),
						'id'       => 'alg_wc_wholesale_pricing_levels_number_' . $user_role_key,
						'default'  => 1,
						'type'     => 'number',
						'desc'     => apply_filters( 'alg_wc_wholesale_pricing', sprintf(
							'<p>You will need <a target="_blank" href="%s">Wholesale Pricing for WooCommerce Pro</a> plugin to add more than one level.</p>',
							'https://wpfactory.com/item/wholesale-pricing-woocommerce/' ), 'settings_number_of_levels' ),
						'custom_attributes' => apply_filters( 'alg_wc_wholesale_pricing', array( 'readonly' => 'readonly' ), 'settings_array' ),
					),
				) );
				for ( $i = 1; $i <= alg_wc_wh_pr_get_total_levels( false, 0, '_' . $user_role_key ); $i++ ) {
					$user_roles_settings = array_merge( $user_roles_settings, array(
						array(
							'title'    => __( 'Min quantity', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' [' . $user_role_key . ']',
							'desc_tip' => __( 'Minimum quantity to apply discount.', 'wholesale-pricing-woocommerce' ),
							'id'       => 'alg_wc_wholesale_pricing_level_min_qty_' . $user_role_key . '_' . $i,
							'default'  => 0,
							'type'     => 'number',
							'custom_attributes' => array('step' => '1', 'min' => '0', ),
						),
						array(
							'title'    => __( 'Discount', 'wholesale-pricing-woocommerce' ) . ' #' . $i . ' [' . $user_role_key . ']',
							'desc_tip' => __( 'To set fee instead of discount - enter negative number.', 'wholesale-pricing-woocommerce' ),
							'id'       => 'alg_wc_wholesale_pricing_level_discount_' . $user_role_key . '_' . $i,
							'default'  => 0,
							'type'     => 'number',
							'custom_attributes' => array('step' => '0.0001' ),
						),
					) );
				}
			}
		}
		$user_roles_settings = array_merge( $user_roles_settings, array(
			array(
				'type'     => 'sectionend',
				'id'       => 'alg_wc_wholesale_pricing_by_user_role_options',
			),
		) );
		return array_merge( $main_settings, $general_settings, $levels_settings, $user_roles_settings );
	}

}

endif;

return new Alg_WC_Wholesale_Pricing_Settings_General();
